% data = FTS_timestream(offset, speed);
%
%   Matthew Hasselfield - Dec. 2, 2008
%
% This function depends on motor_op.m and capture_op.m, used to
% control the Thorlabs motor stage and the video capture card
% respectively.
%
% Performs a Fourier Transform Spectroscopy move, acquiring
% time-stream data from some small area of the video frame.
%
% The FTS move sequence (using arguments 'offset' and 'speed'):
%  - left by 'offset', quickly
%  - right by 2*'offset', at velocity 'speed' while acquiring
%      a video stream
%  - left to return to starting position, quickly
%
% This program should be called with the motor stage centered
% on the region of interest.
%

function data = FTS_timestream(offset, speed)

% Defines
frame_rate = 30.;     % Camera frame rate
fast_speed = 0.1;     % Motor 'goto' speed for fast moves.
window_dim = [10 10]; % Region of interest size
accn = 0.4;           % Reasonable acceleration...

disp('Initializing motor...')
m = motor_op(0, 'init');

disp('Initializing capture card...')
v = capture_op(0, 'init');

% Region select
im = capture_op(v, 'snap');
fig = figure;
imshow(im);
disp('USER: select a region for FTS sampling.')
region = ginput(1);
close(fig);
capture_op(v, 'window', [region window_dim]);
pause(0.5);

disp('Moving to start position...')
start_pos = motor_op(m, 'pos');
motor_op(m, 'set_vel', fast_speed);
motor_op(m, 'goto_wait', start_pos + offset);

disp('Starting FTS move...');
distance = 2*offset;
total_time = 2*offset / speed;
motor_op(m, 'set_vel', speed, speed);
motor_op(m, 'goto', start_pos - distance);

% It seems to take at least a whole second for the motor to start moving.
wait_time = speed / motor_op(m, 'get_accel') + 2.;
pause(wait_time);

% Capture at least the right number of frames
n_frames = (total_time - wait_time - 1) * frame_rate;
disp(['Capturing ' num2str(n_frames) ' frames...'])
capture_op(v, 'start');
capture_op(v, 'frame_count', n_frames);
capture_op(v, 'stop');

% Get them, average them
data = capture_op(v, 'frames', n_frames);
data = squeeze(sum(sum(data)));

disp('Going home')
motor_op(m, 'set_vel', fast_speed, 0);
motor_op(m, 'goto_wait', start_pos);

disp('Cleaning up...')
capture_op(v, 'cleanup');
motor_op(m, 'cleanup');

end
