% capture_op - provides basic control of a video capture card
%
% Matthew Hasselfield - Dec. 2, 2008
%
% Brief usage:
%   Initialization:
%       v = capture_op(0, 'init');
%
%   Clean-up (should be run prior to re-initialization):
%       capture_op(v, 'cleanup');
%
%   Single frame capture:
%       image = capture_op(v, 'snap');
%
%   Video capture:
%     First set the region of interest:
%       capture_op(v, 'window', [x y width height]);
%
%     Then capture some number of seconds of video:
%       frames = capture_op(v, 'capture', time);
%
%     Alternately, start video acquisition, wait for some number of
%     frames to be captured, stop acquisition, and grab the data:
%       capture_op(v, 'start');
%       capture_op(v, 'frame_count', n_frames);
%       capture_op(v, 'stop');
%       frames = capture_op(v, 'frames', n_frames)
%

function output = capture_op(handle, cmd, varargin)

    vid = handle;
    n_argin = size(varargin,2);

    switch(cmd)
        case 'init'
            %%initialization of video card
            info=imaqhwinfo;
            info=imaqhwinfo('winvideo');
            dev_info = imaqhwinfo('winvideo',1);
            vid = videoinput('winvideo',1,'RGB24_752x480');
            set(vid,'SelectedSourceName','composite');
            set(vid,'TriggerRepeat',Inf);
            set(vid,'ReturnedColorSpace','grayscale');
            set(vid,'Timeout',5);   % In seconds...
            output = vid;

        case 'snap'
	    % Capture a single frame of video.  A delay is necessary
            % because when capture is started the gain of the video
            % processor takes a moment to settle down.

            start(vid);
            % Pause for 1 second, unless delay passed as argument.
            if (n_argin >= 1)
                pause(varargin{1});    % For gain to settle :P
            else
                pause(1)
            end
            output = getsnapshot(vid);
            stop(vid);
            
        case 'preview'
            on = 1
            if (n_argin >= 1)
                on = varargin{1}
            end
            if (on ~= 0)
                preview(vid);
            else
                stoppreview(vid);
            end
            
        case 'window'
	    % Pass a 4 element vector [x y width height] to set the
            % section of the video frame that is captured.  (Capturing
            % more than a few frames of full-frame video is a major
	    % system burden.)

            set(vid,'ROIPosition', varargin{1})
            
        case 'capture'
            t = varargin{1};
            n = t * 30;
            set(vid, 'Timeout', t*1.1);
            start(vid);
            [data, times] = getdata(vid, n);
            output = [times, squeeze(sum(sum(data)))];
            stop(vid)

        case 'start'
            % This resets the frames in memory and starts acquisition in
            % the capture card.
            start(vid);
            
        case 'stop'
            % This stops frame capture and releases the video device.  It
            % should be callsed after 'start', as soon as you've acquired
            % all the frames you want. (see 'frame_count') 
            stop(vid);
            
        case 'frame_count'
            % Get 'frames available'.  If argument is given, will block
            % until at least that many frames are acquired.
            block_count = 0;
            if (n_argin >= 1)
                block_count = varargin{1};
            end
            frames = get(vid, 'FramesAvailable');
            while (frames < block_count)
                pause(0.5);
                frames = get(vid, 'FramesAvailable');
            end
            output = frames;

        case 'frames'
            % Obtain some number of frames from the capture card.  This
            % is meant to be used with 'start' and 'frame_count' commands,
            % i.e. to obtain some number of frames that have already been
            % captured following a 'start' command.  To perform a complete
            % acquisition sequence, use 'capture'.

            output = getdata(vid, varargin{1});

        case 'cleanup'
            % Clean up; the capture card will reject future 'init'
            % attempts if this is not done.
            stoppreview(vid);
            stop(vid);
            delete(vid);
            clear vid;
            
        otherwise
            disp(['Unknown command: ' cmd]);
            output = -1;
    end

end
